@@ -9,7 +9,8 @@ easy: you just have to create a plugin for this.
 relatorio also provides a report repository allowing you to link python objects
 and report together, find reports by mimetypes/name/python objects.
 """
-from relatorio.reporting import MIMETemplateLoader, ReportRepository, Report
-import templates
+from .reporting import MIMETemplateLoader, ReportRepository, Report
+from . import templates
 
-__version__ = '0.5.6'
+__version__ = '0.6.1'
+__all__ = ['MIMETemplateLoader', 'ReportRepository', 'Report', 'templates']
