@@ -1,6 +1,6 @@
 ###############################################################################
 #
-# Copyright (c) 2009 Cedric Krier.
+# Copyright (c) 2009-2014 Cedric Krier.
 # Copyright (c) 2007, 2008 OpenHex SPRL. (http://openhex.com) All Rights
 # Reserved.
 #
@@ -31,12 +31,10 @@ except ImportError:
 import time
 import urllib
 import zipfile
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+from io import BytesIO
 from copy import deepcopy
 import datetime
+from decimal import Decimal
 
 
 import warnings
@@ -51,7 +49,6 @@ from genshi.filters.transform import ENTER, EXIT
 from genshi.core import Stream
 from genshi.template.interpolation import PREFIX
 
-#from relatorio import __version__ as VERSION
 import relatorio
 from relatorio.templates.base import RelatorioStream
 from relatorio.reporting import Report, MIMETemplateLoader
@@ -107,7 +104,7 @@ EtreeElement = lxml.etree.Element
 def guess_type(val):
     if isinstance(val, (str, unicode)):
         return 'string'
-    elif isinstance(val, (int, float, long)):
+    elif isinstance(val, (int, float, long, Decimal)):
         return 'float'
 
 
@@ -129,6 +126,8 @@ class ImageHref:
             bitstream = bitstream(**self.context).render()
         elif isinstance(bitstream, ChartTemplate):
             bitstream = bitstream.generate(**self.context).render()
+        elif not hasattr(bitstream, 'seek') or not hasattr(bitstream, 'read'):
+            bitstream = BytesIO(bitstream)
         bitstream.seek(0)
         file_content = bitstream.read()
         name = md5(file_content).hexdigest()
@@ -220,7 +219,7 @@ def update_py_attrs(node, value):
     if not value:
         return
     py_attrs_attr = '{%s}attrs' % GENSHI_URI
-    if not py_attrs_attr in node.attrib:
+    if py_attrs_attr not in node.attrib:
         node.attrib[py_attrs_attr] = value
     else:
         node.attrib[py_attrs_attr] = \
@@ -235,6 +234,7 @@ class Template(MarkupTemplate):
         self.namespaces = {}
         self.inner_docs = []
         self.has_col_loop = False
+        self._source = None
         super(Template, self).__init__(source, filepath, filename, loader,
                                        encoding, lookup, allow_exec)
 
@@ -243,7 +243,21 @@ class Template(MarkupTemplate):
 
         It adds genshi directives and finds the inner docs.
         """
-        zf = zipfile.ZipFile(self.filepath)
+        if not self.filepath:
+            if hasattr(source, 'read') and hasattr(source, 'mode'):
+                if 'U' in source.mode:
+                    # TemplateLoader of Genshi <= 0.6 open files with universal
+                    # newlines which is not suitable for zipfile
+                    raise ValueError('filepath is required '
+                        'if source is openned with universal newlines')
+                else:
+                    # source could be closed before generate calls
+                    source = BytesIO(source.read())
+        else:
+            source = self.filepath
+        self._source = source
+        self.filepath = None  # Prevent zip content in traceback
+        zf = zipfile.ZipFile(source)
         content = zf.read('content.xml')
         styles = zf.read('styles.xml')
 
@@ -265,8 +279,8 @@ class Template(MarkupTemplate):
                                        encoding)
             content_files.append((c_path, c_parsed))
             styles_files.append((s_path, s_parsed))
-
         zf.close()
+
         parsed = []
         for fpath, fparsed in content_files + styles_files:
             parsed.append((genshi.core.PI, ('relatorio', fpath), None))
@@ -277,7 +291,7 @@ class Template(MarkupTemplate):
     def insert_directives(self, content):
         """adds the genshi directives, handle the images and the innerdocs.
         """
-        tree = lxml.etree.parse(StringIO(content))
+        tree = lxml.etree.parse(BytesIO(content))
         root = tree.getroot()
 
         # assign default/fake namespaces so that documents do not need to
@@ -304,7 +318,7 @@ class Template(MarkupTemplate):
         self._handle_images(tree)
         self._handle_innerdocs(tree)
         self._escape_values(tree)
-        return StringIO(lxml.etree.tostring(tree))
+        return BytesIO(lxml.etree.tostring(tree))
 
     def _invert_style(self, tree):
         "inverts the text:a and text:span"
@@ -344,7 +358,7 @@ class Template(MarkupTemplate):
                 raise OOTemplateError("No expression in the tag",
                                       self.filepath)
             closing, directive, attr, attr_val = \
-                    GENSHI_EXPR.match(expr).groups()
+                GENSHI_EXPR.match(expr).groups()
             is_opening = closing != '/'
 
             warn_msg = None
@@ -419,7 +433,7 @@ class Template(MarkupTemplate):
                         pass
                     o_ancestors.append(node)
                 assert ancestor is not None, \
-                       "No common ancestor found for opening and closing tag"
+                    "No common ancestor found for opening and closing tag"
 
                 outermost_o_ancestor = o_ancestors[-1]
                 outermost_c_ancestor = c_ancestors[-1]
@@ -560,7 +574,7 @@ class Template(MarkupTemplate):
             for tag in to_split:
                 tag_pos = table_node.index(tag)
                 num = int(tag.attrib.pop(table_num_col_attr))
-                new_tags = [deepcopy(tag) for _ in range(num)]
+                new_tags = [deepcopy(tag) for i in range(num)]
                 table_node[tag_pos:tag_pos + 1] = new_tags
 
             # compute the column header nodes corresponding to
@@ -571,10 +585,10 @@ class Template(MarkupTemplate):
             # add a <relatorio:repeat> node around the column
             # definitions nodes
             attribs = {
-               "opening": str(opening_pos),
-               "closing": str(closing_pos),
-               "table": table_name
-            }
+                "opening": str(opening_pos),
+                "closing": str(closing_pos),
+                "table": table_name,
+                }
             repeat_node = EtreeElement(repeat_tag, attrib=attribs,
                                        nsmap={'relatorio': RELATORIO_URI})
             wrap_nodes_between(first, last, repeat_node)
@@ -690,7 +704,7 @@ class Template(MarkupTemplate):
 
     def generate(self, *args, **kwargs):
         "creates the RelatorioStream."
-        serializer = OOSerializer(self.filepath)
+        serializer = OOSerializer(self._source)
         kwargs['__relatorio_make_href'] = ImageHref(serializer.outzip,
                                                     serializer.manifest,
                                                     kwargs)
@@ -750,13 +764,17 @@ class DuplicateColumnHeaders(object):
 class Manifest(object):
 
     def __init__(self, content):
-        self.tree = lxml.etree.parse(StringIO(content))
+        self.tree = lxml.etree.parse(BytesIO(content))
         self.root = self.tree.getroot()
         self.namespaces = self.root.nsmap
 
     def __str__(self):
-        return lxml.etree.tostring(self.tree, encoding='UTF-8',
-                                   xml_declaration=True)
+        val = lxml.etree.tostring(self.tree, encoding='UTF-8',
+                                  xml_declaration=True)
+        # In Python 3, val will be bytes
+        if not isinstance(val, str):
+            return str(val, 'utf-8')
+        return val
 
     def add_file_entry(self, path, mimetype=None):
         manifest_namespace = self.namespaces['manifest']
@@ -780,7 +798,7 @@ class Manifest(object):
 class Meta(object):
 
     def __init__(self, content):
-        self.tree = lxml.etree.parse(StringIO(content))
+        self.tree = lxml.etree.parse(BytesIO(content))
         root = self.tree.getroot()
         self.namespaces = root.nsmap
         path = '/office:document-meta/office:meta'
@@ -791,7 +809,7 @@ class Meta(object):
         meta = self.office_meta.find('{%s}%s' % (namespace, name))
         if meta is None:
             meta = EtreeElement('{%s}%s' % (namespace, name),
-                                            nsmap={'meta': namespace})
+                                nsmap={'meta': namespace})
             self.office_meta.append(meta)
         meta.text = value
 
@@ -814,17 +832,21 @@ class Meta(object):
         self.remove('printed-by')
         self.remove('creator', 'dc')
         self.remove('date', 'dc')
-        return lxml.etree.tostring(self.tree, encoding='UTF-8',
-                                   xml_declaration=True)
+        val = lxml.etree.tostring(self.tree, encoding='UTF-8',
+                                  xml_declaration=True)
+        # In Python 3, val will be bytes
+        if not isinstance(val, str):
+            return str(val, 'utf-8')
+        return val
 
 
 class OOSerializer:
 
-    def __init__(self, oo_path):
-        self.inzip = zipfile.ZipFile(oo_path)
+    def __init__(self, source):
+        self.inzip = zipfile.ZipFile(source)
         self.manifest = Manifest(self.inzip.read(MANIFEST))
         self.meta = Meta(self.inzip.read(META))
-        self.new_oo = StringIO()
+        self.new_oo = BytesIO()
         self.outzip = zipfile.ZipFile(self.new_oo, 'w')
         self.xml_serializer = genshi.output.XMLSerializer()
 
@@ -848,7 +870,8 @@ class OOSerializer:
                 new_info = zipfile.ZipInfo(f_info.filename, now)
                 for attr in ('compress_type', 'flag_bits', 'create_system'):
                     setattr(new_info, attr, getattr(f_info, attr))
-                serialized_stream = output_encode(self.xml_serializer(stream))
+                serialized_stream = output_encode(self.xml_serializer(stream),
+                    encoding='utf-8')
                 self.outzip.writestr(new_info, serialized_stream)
             elif f_info.filename == MANIFEST:
                 manifest_info = f_info
