@@ -1,5 +1,6 @@
 ###############################################################################
 #
+# Copyright (c) 2014 Cedric Krier.
 # Copyright (c) 2007, 2008 OpenHex SPRL. (http://openhex.com) All Rights
 # Reserved.
 #
@@ -20,7 +21,7 @@
 
 
 import os
-from nose.tools import *
+from nose.tools import assert_true, eq_, assert_raises
 
 from relatorio.reporting import (ReportRepository, Report, MIMETemplateLoader,
                                  DefaultFactory, _absolute, _guess_type)
@@ -53,7 +54,8 @@ class TestRepository(object):
         assert_true(report.fpath.endswith(os.path.join('templates',
                                                        'test.tmpl')))
 
-        report2, name = reporting.classes[StubObject].mimetypes['text/plain'][0]
+        report2, name = (reporting.classes[StubObject]
+                         .mimetypes['text/plain'][0])
         eq_(name, 'default')
         eq_(report, report2)
 
@@ -98,7 +100,7 @@ class TestReport(object):
                 d['o'] = o
                 d['y'] = y
                 d['time'] = time
-                d['func'] = lambda x: x+1
+                d['func'] = lambda x: x + 1
                 return d
 
         our_dir, _ = os.path.split(__file__)
@@ -112,6 +114,7 @@ class TestReport(object):
             "Hi Foo,\nIt's One o'clock to 5 !\n")
         assert_raises(TypeError, report, a)
 
+
 class TestReportInclude(object):
 
     def test_include(self):
